import 'package:flutter/services.dart';

import 'ecp_sync_plugin_platform_interface.dart';

class EcpSyncPlugin {
  // static const MethodChannel _channel = MethodChannel('ecp_sync_plugin');
  static EcpSyncPlugin? _instance;
  final MethodChannel? _methodChannel;
  final EventChannel? _eventChannel;
  Stream<Map>? _onBatteryStateChanged;

  EcpSyncPlugin.private(this._methodChannel, this._eventChannel);

  factory EcpSyncPlugin() {
    if (_instance == null) {
      const MethodChannel methodChannel = MethodChannel('ecp_sync_plugin');
      const EventChannel eventChannel =
          EventChannel('ecp_sync_plugin_Progress');
      _instance = EcpSyncPlugin.private(methodChannel, eventChannel);
    }
    return _instance!;
  }

  Future<String?> getPlatformVersion() {
    return EcpSyncPluginPlatform.instance.getPlatformVersion();
  }

  Future<String?> syncronizeData(
      {String? userName,
      String? fkEmpGlCode,
      String? urlUpload,
      String? iMei,
      String? appVersion,
      String? uploadMethod,
      String? clientName,
      String? databaseName,
      String? packageName,
      String? dbPassword,
      String? isDbEncripted,
      String? methodSyncID,
      String? fkCustomerGlCode,
      String? DeviceSyncStatus,
      String? CustomerCode,
      String? SystemName,
      String? LastSyncDate,
      String? ServerSyncId,
      String? serviceAPIKey}) async {
    final String version =
        await _methodChannel!.invokeMethod('syncronizeData', {
      'UserName': userName,
      'fk_EmpGlCode': fkEmpGlCode,
      'ClientName': clientName,
      'imei': iMei,
      'version': appVersion,
      'uploadMethod': uploadMethod,
      'urlUpload': urlUpload,
      'databaseName': databaseName,
      'packageName': packageName,
      'dbPassword': dbPassword,
      'isDbEncripted': isDbEncripted,
      'methodSyncID': methodSyncID,
      'fkCustomerGlCode': fkCustomerGlCode,
      'DeviceSyncStatus': DeviceSyncStatus,
      'CustomerCode': CustomerCode,
      'SystemName': SystemName,
      'LastSyncDate': LastSyncDate,
      'ServerSyncId': ServerSyncId,
      'serviceAPIKey': serviceAPIKey
    });

    return version;
  }

  Future<String> syncronizeDataFile(
      {String? userName,
      String? fkEmpGlCode,
      String? urlUpload,
      String? iMei,
      String? appVersion,
      String? uploadMethod,
      String? logUploadMethod,
      String? clientName,
      String? databaseName,
      String? packageName,
      String? dbPassword,
      String? isDbEncripted,
      String? methodSyncID,
      String? fkCustomerGlCode,
      String? DeviceSyncStatus,
      String? CustomerCode,
      String? SystemName,
      String? LastSyncDate,
      String? ServerSyncId,
      String? appVersionCode,
      String? serviceAPIKey}) async {
    final String version =
        await _methodChannel!.invokeMethod('syncronizeDataFile', {
      'UserName': userName,
      'fk_EmpGlCode': fkEmpGlCode,
      'ClientName': clientName,
      'imei': iMei,
      'uploadMethod': uploadMethod,
      'logUploadMethod': logUploadMethod,
      'urlUpload': urlUpload,
      'databaseName': databaseName,
      'packageName': packageName,
      'dbPassword': dbPassword,
      'isDbEncripted': isDbEncripted,
      'methodSyncID': methodSyncID,
      'fkCustomerGlCode': fkCustomerGlCode,
      'DeviceSyncStatus': DeviceSyncStatus,
      'CustomerCode': CustomerCode,
      'SystemName': SystemName,
      'LastSyncDate': LastSyncDate,
      'ServerSyncId': ServerSyncId,
      'appVersion': appVersion,
      'appVersionCode': appVersionCode,
      'serviceAPIKey': serviceAPIKey
    });
    return version;
  }

  Future<String> encodeData(String details, String key, String iv) async {
    final String version = await _methodChannel!
        .invokeMethod('encodeData', {'details': details, 'key': key, 'iv': iv});

    return version;
  }

  Future<String> decodeData(String details, String key, String iv) async {
    final String version = await _methodChannel!
        .invokeMethod('decodeData', {'details': details, 'key': key, 'iv': iv});

    return version;
  }

  Future<String> reEncryption(String details, String key, String iv) async {
    final String version = await _methodChannel!.invokeMethod(
        'RE_Encryption', {'details': details, 'key': key, 'iv': iv});

    return version;
  }

  Future<String> reDescription(String details, String key, String iv) async {
    final String version = await _methodChannel!.invokeMethod(
        'RE_Description', {'details': details, 'key': key, 'iv': iv});

    return version;
  }

  Future<String> startScanChiperLab() async {
    final String version =
        await _methodChannel!.invokeMethod('startScanDeviceChiperLab');

    return version;
  }

  Future<String> stopScanChiperLab() async {
    final String version =
        await _methodChannel!.invokeMethod('stopScanDeviceChiperLab');

    return version;
  }

  Future<String> hideKeyboardPlugin() async {
    final String version =
        await _methodChannel!.invokeMethod('hideKeyboardPlugin');

    return version;
  }

  Future<String> scanBarCodes(
      String? isSingleScan,
      String? isPlaySound,
      String? isOnFlase,
      String? themeColor,
      String? isGoogleVision,
      String? nextBtn,
      String? scanText) async {
    final String version = await _methodChannel!.invokeMethod('scanBarCodes', {
      'isGoogleVision': isGoogleVision,
      'isSingleScan': isSingleScan,
      'isPlaySound': isPlaySound,
      'isOnFlase': isOnFlase,
      'themeColor': themeColor,
      'nextBtn': nextBtn,
      'scanText': scanText
    });

    return version;
  }

  Future<String?> RE_SyncronizeStart(
      {bool? isTransactionSync,
        bool? isPointCalculationSync,
        bool? isMasterSync,
        String? baseURL,
        String? userID,
        String? appVersion,
        String? appVersionCode,
        String? password,
        String? token,
        String? countryID,
        String? role_code,
        String? customer_level,
        String? CFA_DISPATCH,
        String? CFA_STOCK_TRANSFER,
        String? DIST_DISPATCH_MODEL,
        String? DIST_DISPATCH_MODEL_BASIC_SCAN,
        String? DIST_DISPATCH_MODEL_BASIC_SCAN_IDS}) async {
    final String version =
    await _methodChannel!.invokeMethod('RE_CPM_Sync_Start', {
      'isTransactionSync': isTransactionSync,
      'isPointCalculationSync': isPointCalculationSync,
      'isMasterSync': isMasterSync,
      'baseURL': baseURL,
      'userID': userID,
      'appVersion': appVersion,
      'appVersionCode': appVersionCode,
      'password': password,
      'token': token,
      'countryID': countryID,
      'role_code': role_code,
      'customer_level': customer_level,
      'CFA_DISPATCH': CFA_DISPATCH,
      'CFA_STOCK_TRANSFER': CFA_STOCK_TRANSFER,
      'DIST_DISPATCH_MODEL': DIST_DISPATCH_MODEL,
      'DIST_DISPATCH_MODEL_BASIC_SCAN': DIST_DISPATCH_MODEL_BASIC_SCAN,
      'DIST_DISPATCH_MODEL_BASIC_SCAN_IDS': DIST_DISPATCH_MODEL_BASIC_SCAN_IDS
    });

    return version;
  }

  Future<String> getUniqueNumber() async {
    final String version = await _methodChannel!.invokeMethod('UniqueNumber');

    return version;
  }

  Future<bool> checkIfTransactionExistsNew() async {
    final bool version = await _methodChannel!.invokeMethod('RE_CPM_Check_Transaction');
    return version;
  }
  
  Future<String> checkInternet() async {
    final String version = await _methodChannel!.invokeMethod('checkInternet');

    return version;
  }

  Future<String> restorePreferences() async {
    final String version =
        await _methodChannel!.invokeMethod('restorePreferences');

    return version;
  }

  Future<String> openDateSettings() async {
    final String version =
        await _methodChannel!.invokeMethod('openDateSettings');

    return version;
  }

  Future<String> checkBarCode(String code) async {
    final String version =
        await _methodChannel!.invokeMethod('checkBarCode', {'code': code});

    return version;
  }

  Future<String> pickFileFromGallary(
      String plateform, String numberOfFiles) async {
    if (plateform.toLowerCase() == "IOS".toLowerCase()) {
      return "not allowd";
    }

    final String version = await _methodChannel!
        .invokeMethod('pickFileFromGallary', {'numberOfFiles': numberOfFiles});

    return version;
  }

  Future<String> checkSqliteChiper(
      String androidDBPath, String flutterDBPath) async {
    final String version = await _methodChannel!.invokeMethod(
        'checkSqliteChiper',
        {'androidDBPath': androidDBPath, 'flutterDBPath': flutterDBPath});

    return version;
  }

  Future<String> checkPermission(String plateform) async {
    if (plateform.toLowerCase() == "IOS".toLowerCase()) {
      return "not allowd";
    }

    final String version =
        await _methodChannel!.invokeMethod('checkPermission');
    return version;
  }

  Future<String> getTimeFromUTC(String date, String format) async {
    final String version = await _methodChannel!
        .invokeMethod('getTimeFromUTC', {'date': date, 'format': format});

    return version;
  }

  Stream<Map> get onBatteryStateChanged {
    _onBatteryStateChanged ??= _eventChannel!
        .receiveBroadcastStream()
        .map((dynamic event) => _parseBatteryState(event));
    return _onBatteryStateChanged!;
  }

  Map _parseBatteryState(Map state) {
    print("_parseBatteryState : $state");
    return state;
  }
}
